/*
   Allocate some memory, fill it, and report its location.

   Author: Kees Cook <keescook@chromium.org>
   Copyright 2011 ChromeOS Authors
   Author: Steve Beattie <steve.beattie@canonical.com>
   Copyright 2020 Canonical, Ltd.

   make -C /usr/src/linux-headers-$(uname -r) M=$PWD modules
*/

#define pr_fmt(fmt) KBUILD_MODNAME ": " fmt

#include <linux/module.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/slab.h>
#include <asm/io.h>

/* qa-regression-testing common compatability definitions */
#include <qrt-compat.h>

int *signpost;

static int signpost_virt_show(struct seq_file *m, void *v)
{
    seq_printf(m, "%p\n", signpost);
    return 0;
}

static int signpost_virt_proc_open(struct inode *inode, struct file *file)
{
        return single_open(file, signpost_virt_show, NULL);
}

static const struct PROC_OPS signpost_virt_proc_fops = {
        .PROC_OPS_OPEN    = signpost_virt_proc_open,
        .PROC_OPS_READ    = seq_read,
        .PROC_OPS_LSEEK   = seq_lseek,
        .PROC_OPS_RELEASE = single_release,
};

static int signpost_phys_show(struct seq_file *m, void *v)
{
    seq_printf(m, "%08lx\n", (unsigned long)virt_to_phys(signpost));
    return 0;
}

static int signpost_phys_proc_open(struct inode *inode, struct file *file)
{
        return single_open(file, signpost_phys_show, NULL);
}

static const struct PROC_OPS signpost_phys_proc_fops = {
        .PROC_OPS_OPEN    = signpost_phys_proc_open,
        .PROC_OPS_READ    = seq_read,
        .PROC_OPS_LSEEK   = seq_lseek,
        .PROC_OPS_RELEASE = single_release,
};

static int signpost_value_show(struct seq_file *m, void *v)
{
    seq_printf(m, "%08x\n", *signpost);
    return 0;
}

static int signpost_value_proc_open(struct inode *inode, struct file *file)
{
        return single_open(file, signpost_value_show, NULL);
}

static const struct PROC_OPS signpost_value_proc_fops = {
        .PROC_OPS_OPEN    = signpost_value_proc_open,
        .PROC_OPS_READ    = seq_read,
        .PROC_OPS_LSEEK   = seq_lseek,
        .PROC_OPS_RELEASE = single_release,
};

static int __init signpost_init(void)
{
    struct proc_dir_entry *entry;

    signpost = kmalloc(sizeof(*signpost), GFP_KERNEL);
    if (!signpost) {
	pr_err("kmalloc failed\n");
        goto failed;
    }
    *signpost = 0xfeedface;

    PROC_CREATE(entry, "signpost_virt", S_IRUGO, &signpost_virt_proc_fops);
    if (!entry) {
	pr_err("proc_create virt failed\n");
        goto free_signpost;
    }

    PROC_CREATE(entry, "signpost_phys", S_IRUGO, &signpost_phys_proc_fops);
    if (!entry) {
	pr_err("proc_create phys failed\n");
        goto free_virt;
    }

    PROC_CREATE(entry, "signpost_value", S_IRUGO, &signpost_value_proc_fops);
    if (!entry) {
	pr_err("proc_create value failed\n");
        goto free_phys;
    }

    return 0;

free_phys:
    remove_proc_entry("signpost_phys", NULL);
free_virt:
    remove_proc_entry("signpost_virt", NULL);
free_signpost:
    kfree(signpost);
failed:
    return -ENOMEM;
}

static void __exit signpost_exit(void)
{
    remove_proc_entry("signpost_virt", NULL);
    remove_proc_entry("signpost_phys", NULL);
    remove_proc_entry("signpost_value", NULL);
    kfree(signpost);
}

module_init(signpost_init);
module_exit(signpost_exit);

MODULE_LICENSE("GPL");
