/*
      setsockopt-sndbuf.c - a simple program for testing setsockopt
      SO_SNDBUF and SO_SNDBUFFORCE as per CVE-2012-6704 and
      CVE-2016-9793.

      Copyright 2016 Canonical, Ltd.
      Author: Steve Beattie <steve.beattie@canonical.com>
*/

#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <sched.h>
#include <fcntl.h>

#define SOCK_MIN_SNDBUF 2048
unsigned int sysctl_wmem_max = 425984;

void usage(char *progname) {
	fprintf(stderr, "Usage: %s [options] <int>\n", progname);
	fprintf(stderr, "Options are:\n");
	fprintf(stderr, "  -d        emit debugging statements\n");
	fprintf(stderr, "  -f        use SO_SNDBUFFORCE instead of SO_SNDBUFF\n");
	fprintf(stderr, "  -u        use a user namespace\n");
	fprintf(stderr, "  -h        this message\n");
}

int my_unshare(int debug) {
	uid_t uid = geteuid();
        gid_t gid = getegid();
	int fd;
	char buf[256];

	if (debug)
		printf("Unsharing namespaces.\n");

	if (unshare(CLONE_NEWUSER) != 0) {
		perror("FAIL: unable to create new user namespace: ");
		return -11;
	}

	if (unshare(CLONE_NEWNET) != 0) {
		perror("FAIL: unable to create new network namespace: ");
		return -11;
	}


	/* need to deny setgroups to do uid/gid maps */
	fd = open("/proc/self/setgroups", O_WRONLY);
	if (fd < 0) {
		perror("FAIL: unable to open setgroups: ");
		return -1;
	}
	write(fd, "deny", strlen("deny"));
	close(fd);

	/* map uid/gid */
	fd = open("/proc/self/uid_map", O_WRONLY);
	if (fd < 0) {
		perror("FAIL: unable to open uid_map: ");
		return -1;
	}
	sprintf(buf, "0 %d 1", uid);
	write(fd, buf, strlen(buf));
	close(fd);

	fd = open("/proc/self/gid_map", O_WRONLY);
	if (fd < 0) {
		perror("FAIL: unable to open gid_map: ");
		return -1;
	}
	sprintf(buf, "0 %d 1", gid);
	write(fd, buf, strlen(buf));
	close(fd);

	return 0;
}

int main(int argc, char* argv[]) {
	int fd;
	int ret;
	int size, newsize;
	int opt;
	socklen_t len;
	int level = SO_SNDBUF;
	int do_force = 0;
	int do_unshare = 0;
	int debug = 0;

	while ((opt = getopt(argc, argv, "dfhu")) != -1) {
	    switch (opt) {
		    case 'f': do_force = 1;
			      break;
		    case 'u': do_unshare = 1;
			      break;
		    case 'd': debug = 1;
			      break;
		    case 'h':
			      usage(argv[0]);
			      return 0;
		    default:
			      usage(argv[0]);
			      return 1;
	    }
	}

	/* require integer argument */
	if (optind + 1 != argc) {
		usage(argv[0]);
		return 1;
	}

	if (do_force) {
		level = SO_SNDBUFFORCE;
		if (debug)
			printf("Using SO_SNDBUFFORCE.\n");
	}

	ret = strtol(argv[optind], NULL, 10);
	if (ret == LONG_MIN || ret == LONG_MAX) {
		perror("FAIL: strtol");
		return 1;
	}
	newsize = ret;

	if (debug) {
		printf("%d > sysctl_wmem_max = %s\n", newsize, (newsize > sysctl_wmem_max) ? "true" : "false");
		printf("%d * 2 = %d\n", newsize, (newsize * 2));
		printf("(%d * 2) < SOCK_MIN_SNDBUF = %s\n", newsize, ((newsize * 2) < SOCK_MIN_SNDBUF) ? "true" : "false");
	}

	if (do_unshare) {
		if (my_unshare(debug) < 0)
			return 1;
	}

	fd = socket(AF_INET, SOCK_STREAM, 0);

	if (fd < 0) {
		perror("FAIL: socket");
		return 1;
	}

	len = sizeof(int);
	ret = getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &size, &len);

	if (ret < 0) {
		perror("FAIL: getsockopt");
		return 1;
	}

	if (debug)
		printf("SO_GETBUF (before): %d\n", size);

	if (debug) {
		if (do_force) {
			printf("Setting SO_SNDBUFFORCE: %d (unsigned %u)\n", newsize, newsize);
		} else {
			printf("Setting SO_SNDBUF: %d (unsigned %u)\n", newsize, newsize);
		}
	}

	ret = setsockopt(fd, SOL_SOCKET, level, &newsize, sizeof(int));

	if (ret < 0) {
		perror("FAIL: setsockopt");
		return 1;
	}

	len = sizeof(int);
	ret = getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &size, &len);

	if (ret < 0) {
		perror("FAIL: getsockopt");
		return 1;
	}

	if (debug)
		printf("SO_GETBUF (after): ");
	printf("%d\n", size);

	return 0;
}

