#!/usr/bin/python2
#
# ***********************************************************************
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# THIS TEST SCRIPT MAY CAUSE YOUR SYSTEM TO PANIC OR HANG IF A TEST FAILS
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# ***********************************************************************
#
#    test-kernel-panic.py quality assurance test script for kernel panics
#    Copyright (C) 2010 Canonical Ltd.
#    Author: Kees Cook <kees@ubuntu.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3,
#    as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# packages required for test to run:
# QRT-Packages: build-essential libkeyutils-dev keyutils
# packages where more than one package can satisfy a runtime requirement:
# +++ NOTE ++++
# gcc-multilib must be an alternate because it does not exist on armel
# for releases older than oneiric
# QRT-Alternates: gcc-multilib
# files and directories required for the test to run:
# QRT-Depends: kernel-panic

'''
    How to run against a clean schroot named 'hardy':
        schroot -c hardy -u root -- sh -c 'apt-get -y install build-essential libc6-dev-i386 && ./test-kernel-panic.py -v'
'''
from __future__ import print_function

import os
import sys
import tempfile
import time
import unittest
import testlib

try:
    from private.qrt.KernelPanic import PrivateKernelPanicTest
except ImportError:
    class PrivateKernelPanicTest(object):
        '''Empty class'''
    print("Skipping private tests", file=sys.stdout)


class KernelPanicTest(testlib.TestlibCase, PrivateKernelPanicTest):
    '''Test my thing.'''

    def setUp(self):
        '''Set up prior to each test_* function'''
        self.fs_dir = os.path.abspath('.')
        os.chdir('kernel-panic')
        os.system("sync")
        os.system("sync")

    def tearDown(self):
        '''Clean up after each test_* function'''
        os.chdir(self.fs_dir)

    def test_too_early_vsyscall(self):
        '''The vsyscall entries are created too early (CVE-2010-0307)'''
        if self.dpkg_arch != 'amd64':
            self._skipped('amd64 only')
            return

        os.chdir('CVE-2010-0307')
        self.assertShellExitEquals(0, ["make", 'clean'])
        self.assertShellExitEquals(0, ["make", 'test'])

    def test_cve_2013_2094(self):
        '''test linux kernel perf out-of-bounds access (CVE-2013-2094)'''
        if self.dpkg_arch != 'amd64':
            self._skipped('test works on amd64 only')
            return

        os.chdir('CVE-2013-2094')
        self.assertShellExitEquals(0, ["make", 'clean'])
        self.assertShellExitEquals(0, ["make", 'test'])

    def test_cve_2015_7550(self):
        '''test linux kernel keyctl race (CVE-2015-7550)'''

        os.chdir('CVE-2015-7550')
        self.assertShellExitEquals(0, ["make", 'clean'])
        self.assertShellExitEquals(0, ["make", 'all'])

        print()
        full = 100000
        started = time.time()
        for i in range(0, full+1):
            if (i % 1000 == 0):
                taken = time.time() - started
                eta = ""
                if i > 0:
                    per_chunk = float(taken) / float(i)
                    remaining = int(per_chunk * full) - taken
                    eta = " (eta: %dmin %dsec)   " % (remaining / 60, remaining % 60)
                sys.stdout.write("\r\t%d/%d%s" % (i, full, eta))
                sys.stdout.flush()
            self.assertShellExitEquals(0, ['./cve-2015-7550'])

    def test_cve_2015_8539(self):
        '''Verify CVE-2015-8539 is fixed'''

        self.assertShellExitEquals(1, ['keyctl', 'request2', 'user', 'user', '', '@u'])
        output = self.assertShellExitEquals(0, ['keyctl', 'add', 'user', 'user', 'a', '@u'], bare_report=True)
        self.assertShellExitEquals(0, ['keyctl', 'unlink', output.strip(), '@u'])

    def test_cve_2015_8539_2(self):
        '''Verify CVE-2015-8539 is fixed (part 2)'''

        expected = 1
        if not self.kernel_at_least('3.13.0'):
            self._skipped('Unfixed in precise/3.2 kernels')
            expected = -9

        self.assertShellExitEquals(1, ['keyctl', 'request2', 'trusted', 'user', '', '@u'])
        output = self.assertShellExitEquals(expected, ['keyctl', 'add', 'trusted', 'user', 'a', '@u'], bare_report=True)

    def test_cve_2016_10208(self):
        '''test kernel with invalid s_first_meta_bg (CVE-2016-10208)'''

        if not self.kernel_at_least('3.13.0'):
            self._skipped('debugfs command is buggy and prevents testing before 14.04')
            return

        self.tmpdir = tempfile.mkdtemp(prefix='testlib-', dir='/tmp')
        self.tmp_mountpoint = tempfile.mkdtemp(prefix='testlib-mountpoint-', dir='/tmp')

        fs_img = os.path.join(self.tmpdir, 'fs.img')
        self.assertShellExitEquals(0, ['fallocate', '-l', '16M', fs_img])
        self.assertShellExitEquals(0, ['mke2fs', '-t', 'ext4', '-O', 'bigalloc,meta_bg,^resize_inode', '-F', fs_img])
        self.assertShellExitEquals(0, ['debugfs', '-w', '-R', 'ssv first_meta_bg 842150400', fs_img])

        # if vulnerable, will hang on loopback mount
        self.assertShellExitEquals(32, ['mount', '-o', 'loop', fs_img, self.tmp_mountpoint])

        # XXX Need to cleanup temp directories
        testlib.recursive_rm(self.tmpdir)
        testlib.recursive_rm(self.tmp_mountpoint)


if __name__ == '__main__':
    # simple
    testlib.require_sudo()  # Needed for CVE-2016-10208 test
    unittest.main()
